<?php
namespace Teknisa\EST\Modules\Lancamentos\Services;

use Zeedhi\Framework\Util\Functions;

class Est11000Entrada {

    protected $newCode;
    protected $Est11000Factory;
    protected $ValidacoesService;

    protected $nrlancestq;
    protected $nrorg;
    protected $currentTime;
    protected $operador;
    protected $filialLogada;
    protected $idativo = 'S';
    protected $retirada = '3';

    public function __construct($newCode, $environment, $Est11000Factory, $ValidacoesService) {
        $this->newCode = $newCode;
        $this->Est11000Factory = $Est11000Factory;
        $this->ValidacoesService = $ValidacoesService;
        $this->operador = $environment->getCdOperador();
        $this->nrorg = $environment->getNrOrg();
        $this->filialLogada = $environment->getCdFilial();
        $this->currentTime = $environment->getCurrentTime()->format("d/m/Y H:i:s");
    }

    public function validateReqComp($lancamento, $itens) {
        if($this->shouldCreateReqComp($itens)) {
            $reqComp = $this->getRequisicaoComplementarAberta($lancamento);

            $this->insertItensForReqComp($reqComp, $lancamento, $itens);
        }
    }

    public function insertItensForReqComp($NRREQUESTO, $lancamento, $itens) {
        $params = [
            "NRREQUESTO" => $NRREQUESTO,
            "CDFILIAL" => $lancamento['CDFILIMOVI'],
            "QTDEVOREQU" => 0,
            "IDSTITREQU" => 'A',
            "CDFILILANC" => null,
            "NRLANCESTQ" => null,
            "NRSEQUITEM" => null,
            "CDFILLANCREQC" => $lancamento['CDFILIAL'],
            "NRLANCREQC" => $lancamento['NRLANCESTQ']
        ];

        foreach ($itens as $item) {
            if(Functions::arrayKeyExists('createReqComp', $item) && $item['createReqComp']) {
                $params['NRSEQITREQC'] = $item['NRSEQUITEM'];
                $params['CDPRODUTO'] = $item['CDPRODUTO'];
                $params['NRLOTEESTQ'] = $item['NRLOTEESTQ'];
                $params['NRSUBLOTE'] = $item['NRSUBLOTE'];
                $params['QTREQUESTO'] = $item['QTLANCREQCOMP'];



                $newCodeParams = implode('', [$lancamento['CDFILIAL'],
                                  $lancamento['NRREQUESTO'],
                                  $item['CDPRODUTO']]);

                $NRSEQITEM = $this->geraNewSequencial('SEQITREQESTO', $newCodeParams, 4);

                $params['NRSEQITEM'] = $NRSEQITEM;

                $this->Est11000Factory->insertItrequestoComplementar($params);
            }
        }
    }

    public function shouldCreateReqComp($itens) {
        foreach ($itens as $item) {
            if(Functions::arrayKeyExists('createReqComp', $item) && $item['createReqComp'])
                return true;
        }

        return false;
    }

    public function getRequisicaoComplementarAberta($lancamento) {
        $params = [
            "CDFILLANCREQC" => $lancamento['CDFILIAL'],
            "NRLANCREQC" => $lancamento['NRLANCESTQ']
        ];
        $reqComp = $this->Est11000Factory->getRequisicaoComplementarAberta($params);

        if(empty($reqComp)) {
            $NRREQUCOMPESTO = $this->geraNewSequencial('NRREQUESTO');

            $params = [
                "CDFILIMOVI" => $lancamento['CDFILIMOVI'],
                "NRREQUESTO" => $lancamento['NRREQUESTO']
            ];
            $reqCompParameters = $this->Est11000Factory->getRequisicaoComplementarParameters($params);

            $params = [
                "CDFILIAL" => $lancamento['CDFILIAL'],
                "NRREQUESTO" => $NRREQUCOMPESTO,
                "DTREQUESTO" => $lancamento['DTLANCESTQ'],
                "DTPREVREQU" => $lancamento['DTLANCESTQ'],
                "DSREQUESTO" => 'Requisição Complementar - Req. Orig.: ' . $lancamento['NRREQUESTO'] . ' Baixa Orig.:  ' . $lancamento['NRLANCESTQ'],
                "CDSETOR" => $reqCompParameters['CDSETOR'],
                "CDCENTCUST" => $reqCompParameters['CDCENTCUST'],
                "CDOPERREQU" => $this->operador,
                "CDALMOXSAID" => $reqCompParameters['CDALMOXSAID'],
                "CDLCESTSAID" => $reqCompParameters['CDLCESTSAID'],
                "CDALMOXENTR" => $reqCompParameters['CDALMOXENTR'],
                "CDLCESTENTR" => $reqCompParameters['CDLCESTENTR'],
                "CDFILLANCREQC" => $lancamento['CDFILIAL'],
                "NRLANCREQC" => $lancamento['NRLANCESTQ']
            ];
            $this->Est11000Factory->insertRetiradaestoComplementar($params);

            return $NRREQUCOMPESTO;
        }

        return $reqComp['NRREQUESTO'];
    }

    /**
     * Essa função é chamda quando está se inserindo uma retirada com uma requisição.
     * Ela tem a finalidade de desabilitar a requisição que foi passada, para que não
     * se possa mais inserir retiradas com aquela requisição que foi passada.
     *
     * @param $lancamento | row com os dados do lançamento
     * @param $itensLancesto | array com rows com os dados de cada item do lançamento
     */
    public function validateRequisicao($lancamento, $itensLancesto) {
        $this->updateItemStatus($lancamento, $itensLancesto);

        $newReqStatus = $this->getProperRequisicaoStatus($lancamento);

        $this->updateRequisicaoStatus($lancamento["CDFILIMOVI"], $lancamento["NRREQUESTO"], $newReqStatus);
    }

    //atualiza os status dos itens de uma requisição na base
    public function updateItemStatus($lancamento, $itensLancesto) {
        $itensRequesto = [];
        foreach($itensLancesto as $itemLancesto) {
            if(isset($itemLancesto['NRREQUESTO'])){    
                if($itemLancesto['NRREQUESTO'] && $itemLancesto['NRSEQITEM']) {
                    $itensRequesto[] = $itemLancesto;
                }
            }
        }

        if($itensRequesto) {
            $this->encerrarItensRequesto($lancamento, $itensRequesto);
        }
    }

    //lança os CDPRODUTOs pra factory pra atualizar os status dos itens para "E"
    public function encerrarItensRequesto($lancamento, $itensRequesto) {
        $params = [
            "IDSTITREQU" => "E",
            "NRLANCESTQ" => $lancamento["NRLANCESTQ"]
        ];

        foreach ($itensRequesto as $item) {
            $params["CDFILILANC"] = $lancamento["CDFILIALLAN"];
            $params["NRSEQITEM"]  = $item["NRSEQITEM"];
            $params["CDFILIAL"]   = $item["CDFILIAL"];
            $params["NRREQUESTO"] = $item["NRREQUESTO"];
            $params["OLD_NRLANCESTQ"] = null;
            $params["CDPRODUTO"]  = $item["CDPRODUTO"];
            $params["NRSEQUITEM"] = $item["NRSEQUITEM"];
            $params["NRORG"]      = $this->nrorg;

            $this->Est11000Factory->updateItrequestoStatus($params);
        }
    }

    public function getPosestDia($filter) {
        $params = ["CDPRODUTO" => $filter["CDPRODUTO"]];
        $CDPRODESTO = $this->Est11000Factory->getProdesto($params)["CDPRODESTO"];

        $params = [
            "CDFILIAL" => $filter["CDFILIAL"],
            "NRORG" => $this->nrorg
        ];
        $INTPCUSTO = $this->Est11000Factory->getCdtipocusto($params)["CDTIPCUSTEST"];
 
        $params = [
            "CDFILIAL" => $filter["CDFILIAL"],
            "CDALMOXARIFE" => $filter["CDALMOXARIFE"],
            "CDLOCALESTOQ" => $filter["CDLOCALESTOQ"],
            "NRLOTEESTQ" => $filter["NRLOTEESTQ"],
            "NRSUBLOTE" => $filter["NRSUBLOTE"],
            "CDPRODESTO" => $CDPRODESTO
        ];
        $DTPOSIESTQ = $this->Est11000Factory->getDtposiestq($params)["DTPOSIESTQ"];

        $params = [
            "CDFILIAL" => $filter["CDFILIAL"],
            "CDALMOXARIFE" => $filter["CDALMOXARIFE"],
            "CDLOCALESTOQ" => $filter["CDLOCALESTOQ"],
            "NRLOTEESTQ" => $filter["NRLOTEESTQ"],
            "NRSUBLOTE" => $filter["NRSUBLOTE"],
            "CDPRODESTO" => $CDPRODESTO,
            "DTPOSIESTQ" => $DTPOSIESTQ,
            "INTPCUSTO" => $INTPCUSTO
        ];
        return $this->Est11000Factory->getPosestdia($params);
    }

    //retorna o status da requisição a partir dos status dos itens da mesma
    public function getProperRequisicaoStatus($lancamento) {
        $params = [
            "CDFILIAL" => $lancamento["CDFILIMOVI"],
            "NRREQUESTO" => $lancamento['NRREQUESTO'],
            "NRORG" => $this->nrorg,
            "IDSTITREQU" => null
        ];
        
        $itensRequesto = $this->Est11000Factory->getItensByRequisicao($params);
        $statusRequesto = $itensRequesto[0]["IDSTITREQU"];
        foreach ($itensRequesto as $itemRequesto) {
            if($itemRequesto["IDSTITREQU"]!= $statusRequesto) {
                $statusRequesto = "P";
            }
        }

        return $statusRequesto;
    }

    //atualiza o status da requisição
    public function updateRequisicaoStatus($CDFILIAL, $NRREQUESTO, $IDSTATUSREQ) {
        $IDREQUESTO = ($IDSTATUSREQ == 'A')? 'P' : 'C';

        $params = [
            "IDREQUESTO" => $IDREQUESTO,
            "IDSTATUSREQ" => $IDSTATUSREQ,
            "CDFILIAL" => $CDFILIAL,
            "NRREQUESTO" => $NRREQUESTO
        ];

        $this->Est11000Factory->updateRequestoStatus($params);
    }

    /**
     * Essa função tem como finalidade editar um item já existente de uma retirada já
     * existente. Só é possivél editar 3 campos do registro de item de uma retirada.
     *
     * @param $item | row com os dados do item a ser editado
     */
    public function editItemOfRetirada($item) {
        $params = array('QTTOTLANCTO' => $item['QTTOTLANCTO'],
                        'QTDEVOLESTOQ' => $item['QTDEVOLESTOQ'],
                        'QTLANCTOEST' => $item['QTLANCTOEST'],
                        'NRLANCESTQ' => $item['NRLANCESTQ'],
                        'CDCENTCUST' => $item['CDCENTCUST'],
                        'CDTIPORETI' => $item['CDTIPORETI'],
                        'NRORG' => $this->nrorg,
                        'NRSEQUITEM' => $item['NRSEQUITEM'],
                        'CDFILIAL' => $item['CDFILIAL']);

        $this->Est11000Factory->updateItlanctoest($params);

        return ['CDFILIAL' => $item['CDFILIAL'], 'NRLANCESTQ' => $item['NRLANCESTQ']];
    }

    /**
     * Essa função tem como finalidade chama a função de novo código, para gerar um novo código
     * conforme a tabala passada.
     *
     * @param $table | string com o nome da tabela de onde se gerara o novo código
     * @param $param | parametro para geração do novo código
     *
     * @return string | valor do código gerado
     */
    public function geraNewSequencial($table, $param = '', $colSize = 10) {
        return $this->newCode->getCodeByTableName($table.$param, $this->nrorg, 1, $colSize);
    }

    /**
     * Essa função é chamada depois de se inserir uma retirada. Ela tem a finalidade de retornar
     * para o frontend os dados de código que foram gerados, e os damais dados que não está no
     * frontend, afim de setar a currentRow, evitando de dar um reload na tela e perder no foco
     * no registro inserido. Dessa forma o usuário pode seguir com as operações na requisição
     * sem problemas.
     *
     * @param $cdfilimovi | código da filial onde foi lançada a retirada
     * @param $dtlancestq | data de lançamento da retirada
     */
    public function getLancamento($cdfilimovi, $dtlancestq) {
        $params = array("CDFILIAL"=>$cdfilimovi,
                        "DATA"=>$dtlancestq,
                        "NRLANCESTQ"=>$this->nrlancestq,
                        "NRORG"=>$this->nrorg);
        return $this->Est11000Factory->getLancamento($params);
    }

    /**
     * Essa função tem como finalidade receber um array com os itens que serão inseridos na tabela
     * ITLANCTOEST. A função percorre os itens passados para ela, montando um array associativo com
     * os parametros que serão passados para a query. Os produtos são inseridos um a um na tabela.
     *
     * @param $itens | array com os itens
     * @param $dtlancestq | data da retirada
     * @param $cdtiporeti | código do Tipo de Operação da retirada
     * @param $cdcentcust | código do centro de custo do Tipo de Operação da retirada
     * @param $nrrequesto | código da requisição (null se não tiver requisição)
     *
     */
    public function insertItensForRetirada($itens, $dtlancestq, $cdtiporeti, $cdcentcust, $nrrequesto) {
        for($i = 0; $i < count($itens); $i++) {
            //Consulta para pegar alguns dados que serão inseridos na tabela ITLANCTOEST.
            $paramProd = $this->Est11000Factory->getParamProduto(
                                $params = array("CDPRODUTO"=>$itens[$i]['CDPRODUTO'],
                                                "NRORG"=>$this->nrorg));
            //Se a row ja tiver NRLANCESTQ, significa que é a inserção de um item novo em uma
            //retirada ja existente. Se não está no processo de inserir uma retirada.
            if(empty($itens[$i]['NRLANCESTQ'])) {
                $itens[$i]['NRLANCESTQ'] = $this->nrlancestq;
            }
            
            $paramsVrbrut = [
                "CDPRODUTO" => $itens[$i]['CDPRODUTO'],
                "CDFILIAL" => $itens[$i]['CDFILIMOVI'],
                "CDALMOXARIFE" => $itens[$i]['CDALMOXARIFE'],
                "CDLOCALESTOQ" => $itens[$i]['CDLOCALESTOQ'],
                "NRLOTEESTQ" => $itens[$i]['NRLOTEESTQ'],
                "NRSUBLOTE" => $itens[$i]['NRSUBLOTE']
            ];
            
            //Montagem dos parametros
            $params = array("CDFILIAL"=>$this->filialLogada,
                            "NRLANCESTQ"=>$itens[$i]['NRLANCESTQ'],
                            "NRSEQUITEM"=>$itens[$i]['NRSEQUITEM'],
                            "CDPRODUTO"=>$itens[$i]['CDPRODUTO'],
                            "DSLANCESTIT"=>null,
                            "CDALMOXARIFE"=>$itens[$i]['CDALMOXARIFE'],
                            "CDLOCALESTOQ"=>$itens[$i]['CDLOCALESTOQ'],
                            "NRLOTEESTQ"=>$itens[$i]['NRLOTEESTQ'],
                            "NRSUBLOTE"=>$itens[$i]['NRSUBLOTE'],
                            "CDFILIMOVI"=>$itens[$i]['CDFILIMOVI'],
                            "CDPRODMOVI"=>$paramProd['CDPRODESTO'],
                            "IDORDEMCLC"=>'9',
                            "DTLANCMOVI"=>$dtlancestq,
                            "IDEVOLESTOQ"=>'S',
                            "CDTIPORETI"=>$cdtiporeti,
                            "CDCENTCUST"=>$cdcentcust,
                            "QTTOTLANCTO"=>$itens[$i]['QTTOTLANCTO'],
                            "IDTIPOMOVI"=>$this->retirada,
                            "QTLANCTOEST"=>($itens[$i]['QTTOTLANCTO'] * $paramProd['VRFATOCONV']),

                            "VRUNILANCTO"=>round($itens[$i]['VRUNILANCTO'],5),
                            "VRTOTLANCTO"=>round($itens[$i]['VRTOTLANCTO'],5),
                            "QTDEVOLESTOQ"=>$itens[$i]['QTDEVOLESTOQ'],
                            "VRLANCTOEST"=>round($itens[$i]['VRUNILANCTO'],5),
                            "VRLANCTOBRUT"=>round($this->getPosestDia($paramsVrbrut)['VRCUSTOPROD'],5),
                            
                            "NRREQUESTO"=>$nrrequesto,
                            "NRORG"=>$this->nrorg,
                            "IDATIVO"=>$this->idativo,
                            "DTULTATU"=>$this->currentTime,
                            "NRORGULTATU"=>$this->nrorg,
                            "CDOPERULTATU"=>$this->operador,
                            "DTINCLUSAO"=>$this->currentTime,
                            "NRORGINCLUSAO"=>$this->nrorg,
                            "CDOPERINCLUSAO"=>$this->operador);

            $this->Est11000Factory->insertItlanctoest($params);
        }

        return ["CDFILIAL" => $this->filialLogada, "NRLANCESTQ" => $itens[0]['NRLANCESTQ']];
    }

    /**
     * Essa função tem como finalidade executar o processo de edição de uma retirada.
     * Para isso é necessário alterar registro em três tabelas diferentes. Na RETIRADESTO,
     * na LANCTOESTO. Em seguida todos os itens daquela retirada são buscado, para que os
     * campos necessários sejam editados (tabela ITLANCTOEST).
     *
     * @param $lancamento | row com so dados da retirada que esta sendo editada
     */
    public function processEditLancamento($lancamento) {
        $paramsToUpdateRetiradaesto = array('CDSETOR' => $lancamento['CDSETOR'],
                        'CDCENTCUST' => $lancamento['CDCENTCUST'],
                        'CDFILIAL' => $lancamento['CDFILIAL'],
                        'NRORG' => $lancamento['NRORG'],
                        'NRRETEST' => $lancamento['NRRETEST']);

        $this->Est11000Factory->updateRetiradaesto($paramsToUpdateRetiradaesto);

        $paramsToUpdateLanctoestoq = array('DSLANCESTQ' => $lancamento['DSLANCESTQ'],
                                           'CDFILIAL' => $lancamento['CDFILIAL'],
                                           'NRORG' => $lancamento['NRORG'],
                                           'NRLANCESTQ' => $lancamento['NRLANCESTQ']);

        $this->Est11000Factory->updateLanctoesto($paramsToUpdateLanctoestoq);

        $paramsGetItens = array('NRLANCESTQ' => $lancamento['NRLANCESTQ'],
                                'CDFILIAL' => $lancamento['CDFILIAL'],
                                'NRORG' => $lancamento['NRORG']);

        $itens = $this->Est11000Factory->getItensFromRetirada($paramsGetItens);

        for ($i = 0; $i < count($itens); $i++) {
            $paramsForUpdateItlanctoest = array('QTTOTLANCTO' => $itens[$i]['QTTOTLANCTO'],
                                                'QTDEVOLESTOQ' => $itens[$i]['QTDEVOLESTOQ'],
                                                'QTLANCTOEST' => $itens[$i]['QTLANCTOEST'],
                                                'CDCENTCUST' => $lancamento['CDCENTCUST'],
                                                'CDTIPORETI' => $lancamento['CDTIPORETI'],
                                                'NRLANCESTQ' => $itens[$i]['NRLANCESTQ'],
                                                'NRORG' => $itens[$i]['NRORG'],
                                                'NRSEQUITEM' => $itens[$i]['NRSEQUITEM'],
                                                'CDFILIAL' => $itens[$i]['CDFILIAL']);

            $this->Est11000Factory->updateItlanctoest($paramsForUpdateItlanctoest);
        }
    }

    /**
     *
     * @param $item | row com dados do item a ser excluido
     */
    public function processDeleteItem($item) {

        $params = [
            "CDFILIAL" => $item["CDFILIAL"],
            "NRLANCTOEST" => $item["NRLANCESTQ"],
            "NRSEQUITEM" => $item["NRSEQUITEM"]
        ];

        $this->Est11000Factory->deleteItlanctoest($params);
    }

    public function getRequisicaoComplementar($row) {
        $params = [
            "CDFILLANCREQC" => $row["CDFILIAL"],
            "NRLANCREQC" => $row["NRLANCESTQ"]
        ];
        return $this->Est11000Factory->getRequisicaoComplementar($params);
    }

    public function processDeleteItemRetiradaComRequisicaoComplementar($item) {
        $reqComp = $this->getRequisicaoComplementar($item);

        if(!empty($reqComp)) {
            $params = $reqComp;
            $params['CDPRODUTO'] = $item['CDPRODUTO'];
            $params['NRLOTEESTQ'] = $item['NRLOTEESTQ'];
            $params['NRSUBLOTE'] = $item['NRSUBLOTE'];

            $itReqComp = $this->getItReqComp($params);

            if(empty($itReqComp)) {
                $this->Est11000Factory->deleteItrequesto($params);
            } else {
                $this->Est11000Factory->freeItemRequisicaoComplementar($params);
            }
        }
    }

    public function getItReqComp($params) {
        $usedItrequesto = $this->Est11000Factory->getUsedItrequesto($params);

        if($usedItrequesto)
            $usedItrequesto = $usedItrequesto[0];

        return $usedItrequesto;
    }

    /**
     *
     * @param $retirada | row com os dados da retirada a ser excluida
     */
    public function processDeleteEntrada($retirada) {

        $params = [
            "CDFILIAL" => $retirada["CDFILIAL"],
            "NRLANCTOEST" => $retirada["NRLANCESTQ"]
        ];

        $this->Est11000Factory->deleteAllItlanctoest($params);
        $this->Est11000Factory->deleteLanctoestoq($params);
    }

    public function processDeleteRetiradaComRequisicaoComplementar($retirada) {
        $reqComp = $this->getRequisicaoComplementar($retirada);

        if(!empty($reqComp)) {
            if($this->requisicaoComplementarIsFree($reqComp)) {
                $this->Est11000Factory->deleteRequesto($reqComp);
            } else {
                $params = [
                    "CDFILIAL" => $retirada["CDFILIAL"],
                    "NRREQUESTO" => $retirada["NRREQUESTO"]
                ];
                $this->Est11000Factory->freeRequisicaoComplementar($params);
            }
        }
    }

    public function requisicaoComplementarIsFree($reqComp) {
        $params = $reqComp;
        $params['CDPRODUTO'] = null;
        $params['NRLOTEESTQ'] = null;
        $params['NRSUBLOTE'] = null;

        $usedReqCompItems = $this->Est11000Factory->getUsedItrequesto($params);

        return empty($usedReqCompItems);
    }

    /**
     * Essa função tem a finalidade de salvar a movimentação realizada na tabela LANCTOESTOQ.
     *
     * @param $dtlancestq | data da movimentação
     * @param $dslancestq | descrição da movimentação
     * @param $cdfilimovi | códugo da filial onde foi realizada a movimentação
     */
    public function saveLanctoestoq($dtlancestq, $dslancestq, $cdfilimovi) {
        $this->nrlancestq = $this->geraNewSequencial('LANCTOEST', $this->filialLogada);
        $params = array("CDFILIAL"=>$this->filialLogada,
                        "CDOPERADOR"=>$this->operador,
                        "IDTPLANCTO"=>$this->retirada,
                        "DTLANCESTQ"=>$dtlancestq,
                        "DSLANCESTQ"=>$dslancestq,
                        "CDFILIMOVI"=>$cdfilimovi,
                        "NRLANCESTQ"=>$this->nrlancestq,
                        "NRORG"=>$this->nrorg,
                        "IDATIVO"=>$this->idativo,
                        "DTULTATU"=>$this->currentTime,
                        "NRORGULTATU"=>$this->nrorg,
                        "CDOPERULTATU"=>$this->operador,
                        "DTINCLUSAO"=>$this->currentTime,
                        "NRORGINCLUSAO"=>$this->nrorg,
                        "CDOPERINCLUSAO"=>$this->operador);
        $this->Est11000Factory->insertLanctoestoq($params);
    }

    /**
     * Essa função é chamada no momento em que está se inserindo uma retirada. Ela tem a função
     * de inserir os registro na tabela RETIRADAESTO.
     *
     * @param $cdfilimovi | código da filial onde está sendo feita a retirada
     * @param $dtlancestq | data da retirada
     * @param $dslancestq | descrição da retirada
     * @param $cdsetor | código do setor passado para a retirada
     * @param $cdcentcust | código do centro de custo
     * @param $nrordprod | null
     * @param $nrrequesto | código da requisção passad (null se não tiver passado)
     */
    public function saveRetiradaesto($cdfilimovi, $dtlancestq, $dslancestq, $cdsetor,
                                     $cdcentcust, $nrordprod, $nrrequesto) {
        $nrretest = $this->geraNewSequencial('RETIESTO', $cdfilimovi);
        $params = array("CDFILIAL"=>$cdfilimovi,
                        "NRRETEST"=>$nrretest,
                        "DTRETEST"=>$dtlancestq,
                        "DSRETEST"=>$dslancestq,
                        "CDFILIALLAN"=>$this->filialLogada,
                        "NRLANCESTQ"=>$this->nrlancestq,
                        "CDSETOR"=>$cdsetor,
                        "CDCENTCUST"=>$cdcentcust,
                        "NRORDPROD"=>$nrordprod,
                        "NRREQUESTO"=>$nrrequesto,
                        "NRORG"=>$this->nrorg,
                        "IDATIVO"=>$this->idativo,
                        "DTULTATU"=>$this->currentTime,
                        "NRORGULTATU"=>$this->nrorg,
                        "CDOPERULTATU"=>$this->operador,
                        "DTINCLUSAO"=>$this->currentTime,
                        "NRORGINCLUSAO"=>$this->nrorg,
                        "CDOPERINCLUSAO"=>$this->operador);
        $this->Est11000Factory->insertRetiradaesto($params);
    }

    /**
     * Essa função tem como finaidade verificar se filial, onde está se executando as ações de
     * edição (inclui também edição/exclusão/inserção de um novo item) inclusão e exclusão,
     * tem inventário pendente, se tiver levanta um ero.
     *
     * @param $cdfilimovi | código da filial que está sendo editada/excluida/inserida
     * @param $dtlancamento | data de movimentação da filial que está sendo editada/excluida/inserida
     */
    public function verificaInventarioRotativo($cdfilimovi, $dtlancamento) {
        $params = array("CDFILIAL"=>$cdfilimovi,
                        "DTLANCAMENTO"=>$dtlancamento,
                        "NRORG"=>$this->nrorg);
        $inventario = $this->Est11000Factory->verificaInventarioRotativo($params);
        if(!empty($inventario)) {
            throw new \Exception("Operação bloqueada! Inventário pendente para a filial.", 1);
        }
    }
}