<?php
namespace Geral\Repositories;

/**
 *
 */
class Empresa extends \Doctrine\ORM\EntityRepository
{

    /**
     * Verifica se a empresa da AF pertence a filial, na importacao de AFs para notas fiscais.
     *
     * @param $cdempresa
     * @param $cdfilial
     * @param $nrorg
     * @return array
     */
    public function empresaPernteceAFilialDaImportacao($cdempresa, $cdfilial, $nrorg)
    {
        $sql = "
            SELECT E.CDEMPRESA, F.CDINSCESTA
              FROM EMPRESA E, FILIAL F
              WHERE E.CDEMPRESA = :P_CDEMPRESA
                AND E.CDEMPRESA = F.CDEMPRESA
                AND E.NRORG     = F.NRORG
                AND F.CDFILIAL  = :P_CDFILIAL
                AND F.NRORG       = :P_NRORG
            UNION ALL
            SELECT E.CDEMPRESA, EC.CDINSCESTA
              FROM EMPRESA E, EMPRCNTRFIL EC
              WHERE E.CDEMPRESA = :P_CDEMPRESA
               AND EC.CDEMPRESA = E.CDEMPRESA
               AND EC.NRORG     = E.NRORG
               AND EC.CDFILIAL  = :P_CDFILIAL
               AND E.NRORG       = :P_NRORG
        ";

        $params = array(
            "P_CDEMPRESA" => $cdempresa,
            "P_CDFILIAL"  => $cdfilial,
            "P_NRORG"     => $nrorg,
        );

        return $this->getEntityManager()->getConnection()->fetchAll($sql, $params);
    }
    
    public function getEstadoPais($cdempresa, $cdinscesta, $nrorg)
    {
    	$sql = "SELECT  I.CDPAIS  ,I.SGESTADO
			    FROM MUNICIPIO MI, INSCRESTAD I, EMPRESA EP
				WHERE
				   I.CDEMPRESA   = :P_CDEMPRESA        AND
				   I.CDINSCESTA  = :P_CDINSCESTA       AND
				   I.CDPAIS      = MI.CDPAIS(+)      AND
				   I.SGESTADO    = MI.SGESTADO(+)    AND
				   I.CDMUNICIPIO = MI.CDMUNICIPIO(+) AND
				   EP.CDEMPRESA  = I.CDEMPRESA AND
	    		   EP.NRORG		 = :P_NRORG";
    
    	$params = array(
    			"P_CDEMPRESA" => $cdempresa,
    			"P_CDINSCESTA"  => $cdinscesta,
    			"P_NRORG"     => $nrorg,
    	);
    
    	return $this->getEntityManager()->getConnection()->fetchAssoc($sql, $params);
    }

    public function maxCdempresa()
    {
        $sqlMax = "SELECT MAX(TO_NUMBER(CDEMPRESA))+1 FROM EMPRESA";
        $result = $this->getEntityManager()->getConnection()->fetchColumn($sqlMax);
        return ($result) ? $result : false;
    }

    public function getMaxCdempresaPorOrganizacao($nrorg){
        $query = "SELECT MAX(TO_NUMBER(CDEMPRESA))+1 AS CDEMPRESA FROM EMPRESA WHERE NRORG = :nrorg";
        $param = array("nrorg" => $nrorg);
        $result = $this->getEntityManager()->getConnection()->fetchAll($query, $param);
        return empty($result[0]['CDEMPRESA']) ? 1 : $result[0]['CDEMPRESA'];
    }

}